# /t/keycuts_writer.rb - Another take on keyboard shortcuts
# Copyright 2009, Martin Rinehart

require 'sketchup'

module Keycuts

html_file = "/sketchup/keyboard_shortcuts.html"

google_shortcuts = <<eos
A Draw/Arc
Ctrl+A Edit/Select All
B Tools/Paint Bucket
Alt+Backspace Edit/Undo
C Draw/Circle
Ctrl+C Edit/Copy
Delete Edit/Delete
Shift+Delete Edit/Cut
E Tools/Eraser
Ctrl+Shift+E Camera/Zoom Extents
F Tools/Offset
Shift+F1 SketchUp/Context Help
G Edit/Make Component...
H Camera/Pan
Ctrl+Insert Edit/Copy
Shift+Insert Edit/Paste
L Draw/Line
M Tools/Move
Ctrl+N File/New
O Camera/Orbit
Ctrl+O File/Open...
P Tools/Push/Pull
Ctrl+P File/Print...
PageDown View/Animation/Next Scene
PageUp View/Animation/Previous Scene
Q Tools/Rotate
R Draw/Rectangle
S Tools/Scale
Ctrl+S File/Save
Space Tools/Select
T Tools/Tape Measure
Ctrl+T Edit/Select None
Ctrl+V Edit/Paste
W Camera/Walk
Ctrl+Shift+W Camera/Zoom Window
Ctrl+X Edit/Cut
Ctrl+Y Edit/Redo
Z Camera/Zoom
Ctrl+Z Edit/Undo
Shift+Z Camera/Zoom Extents
eos
# end of google_shortcuts

$doctype = <<eos
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
   "http://www.w3.org/TR/html4/loose.dtd">
   
<html>
   
eos

$html_header = <<eos 
<!-- keyboard_shortcuts.html -->

<!-- modeled after tutorial_keyboard_shortcuts.html --> 
<!-- from Edges to Rubies - the Complete SketchUp Tutorial -->
<!-- Copyright 2009, Martin Rinehart -->
 
<head> 
	<title> SketchUp Keyboard Shortcuts </title> 
 
	<meta name='description' content='tutorial for Google SketchUp including SketchUp Rubies'> 
 
	<meta name='keywords' content='SketchUp, Google SketchUp, SketchUp Rubies, tutorial'> 
 
	<meta name='author' content='keycuts.rb, by Martin Rinehart'> 
 
	<meta http-equiv="Content-Type" content="text/html;charset=ISO-8859-1" /> 
 
	<style> 
		body { 
			font-size: 11px; 
			font-family: FreeSans, Helvetica, Arial, SansSerif; 
		}
		.g { background: #f0fff0 }
		.w { background: white }
		.new { background: #ffd0d0 }
	</style> 
	
</head> 
 
<body onkeydown=keydown()> 
 
<table bgcolor=#a0a0cc border=0 cellpadding=1> 
	<caption><h4>SketchUp <span class=new>(and Added)</span> Keyboard Shortcuts</h4></caption> 
	<tr> 
		<th> Key </th> 
		<th> Default </th> 
		<th> Shift+Key </th> 
		<th> Ctrl+Key </th> 
		<th> Alt+Key </th> 
	</tr> 
 
eos
# end of html_header

$html_footer = <<eos 

<script type='text/JavaScript'>

	function keydown( e ) { 
		e = e || event
		if ( e.keyCode == 75 ) { location = 'skp:close@' }
	}
	
</script>
 
</body> 
 
</html> 
 
<!-- end of keyboard_shortcuts.html --> 
eos
# end of html_footer

class Queue # a queue of Shortcuts
# a pop operation removes all Shortcuts of a single letter 
# e.g., A, Shift+A and Alt+A

	def initialize( array )
		@array = array
	end

	
	def pop( )
		return nil if @array.length == 0
		ret = []
		letter = @array[0].letter
		while ( @array.length > 0 ) && ( @array[0].letter == letter ) do
			ret.push( @array[0] )
			@array.delete_at( 0 )
		end
		
		return ret 

	end # pop()
	
	def to_s
		return 'Queue: number of Shortcuts=' + @array.length.to_s
	end 
	
end # of class queue

class Shortcut
	attr_accessor :key_part, :command, :modifiers, :letter, :goog_cut

	def initialize( s )
		s =~ /([\S]*)\s(.*)/
		@key_part = $1
		@command = $2
		@key_part =~ /(.*[+])(.*)$/
		if ( $1 )
			@modifiers = $1
			@letter = $2
		else
			@modifiers = ''
			@letter = @key_part
		end
			@goog_cut = false
	end

	def <=> ( your )
		if @letter == your.letter
			return @modifiers <=> your.modifiers 
		else
			return @letter <=> your.letter
		end
	end
	
	def == ( your )
		return false if self.class != your.class # self: find out why!
		return (@letter == your.letter) &&
			( @modifiers = your.modifiers ) &&
			( @command = your.command )
	end
			

	def compound_modifier?
		@modifiers =~ /(.*\+)(.*\+)/
		return $2 != nil
	end

	def to_s
		return 'Shortcut: ' +
		'key=' + @key_part +
		', command=' + @command
	end

end # of class Shortcut

def self.by_letter( scuts )
# returns array of arrays, by letter, for instance:
# [ [A, Alt+A], [B], [D, Shift+D, Ctrl+D], ...
	queue = Queue.new( scuts )
	ret = []
	while true
		t = queue.pop()
		return ret unless t
		ret.push( t )
	end
end # of by_letter()

def self.by_letter_find( scuts_by_letter, letter )
# returns the array of shortcuts matching "letter"
	scuts_by_letter.each do |sbl|
		return sbl if sbl[ 0 ].letter == letter
	end
	return nil
end # by_letter_find()

def self.by_letter_match( all_by_letter, googs_by_letter )
# finds goog that matches all, sets @goog_cut true for match
	all_by_letter.each do |all|
		letter = all[ 0 ].letter
		googs = by_letter_find( googs_by_letter, letter )
		next unless googs
		by_letter_set( all, googs )
	end
end # of by_letter_match

def self.by_letter_set( all, googs )
# all is an array of Shortcuts from all Shortcuts, all with the same letter
# googs is an array of Shortcuts from goog_cuts, also with the same letter
# sets @goog_cut true if "all" Shortcut exactly matches a goog

# before running this code, all @goog_cuts are false

	all.each do |a| 
		googs.each do |g|
			if ( a.key_part == g.key_part ) && ( a.command == g.command )
				a.goog_cut = true;
				break;
			end
		end
	end

end # of by_letter_set()

def self.get_modifier( modi, arr )
# Given a modifier ('', 'Shift+', 'Ctrl+' or 'Alt+') and an array of Shortcuts
# returns the Shortcut with the specified modifier, or nil.

	arr.each do |s| return s if s.modifiers == modi end
	return nil
	
end # get_modifier()

def self.write_html_header

	$text = $doctype
	t = Time.now
	$text += t.strftime( "<!-- Written by \"keycuts.rb\", %d %B, %Y. -->\n\n" )
	$text += $html_header
	
end # write_html_header		

def self.write_table_row( scuts, color )
	
	row = "\t<tr class='" + color + "'>\n"
	row += "\t\t<td align=center> " + scuts[ 0 ].letter + " </td>\n"
	
		[ '', 'Shift+', 'Ctrl+', 'Alt+' ].each do |m|
			scut = get_modifier( m, scuts )
			if ( not scut ) || ( scut.goog_cut )
				row += "\t\t<td> "
			else
				row += "\t\t<td class='new'> "
			end
			row += scut.command if scut
			row += " </td>\n"
		end
	
	row += "\t</tr>\n"

	$text += row

end # write_table_row()

def self.write_table_rows( scuts_by_letter )

	i = 0
	color = "w"
	scuts_by_letter.each do |scuts|
		write_table_row( scuts, color )
		i += 1
		if i == 3
			i = 0
			color = ( color == "w" ) ? "g" : "w"
		end
	end

end # write_table_rows()

def self.write_compound_notes( scuts )

	$text += "\n</table>\n<p>\n"
	return if scuts.length == 0
	
	$text += "Also:<br>\n"
	scuts.each do |s| 
		$text += "&emsp;" + s.key_part + ': ' + s.command + "<br>\n"
	end

end # write_compound_notes( scuts )

def self.write_table_footer()
	$text += $html_footer
end # write_table_footer()
		
arr = Sketchup.get_shortcuts 

shortcuts = []

arr.each do |a|
	shortcuts.push( Shortcut.new(a) )
end

shortcuts.sort!

simple_shortcuts = []
compound_shortcuts = []

shortcuts.each do |s|
	if s.compound_modifier?
		compound_shortcuts.push( s )
	else
		simple_shortcuts.push( s )
	end
end

shortcuts_by_letter = by_letter( simple_shortcuts )

gs = google_shortcuts.split( /\n/ )
goog_cuts = []
gs.each do |g| goog_cuts.push( Shortcut.new(g) ) end
goog_cuts_by_letter = by_letter( goog_cuts )

by_letter_match( shortcuts_by_letter, goog_cuts_by_letter )

$text = ''

write_html_header()
write_table_rows( shortcuts_by_letter )
write_compound_notes( compound_shortcuts ) 
write_table_footer()


begin
	f = File.new( html_file, "w" )
	f.write( $text )
	f.close()

	UI.messagebox( html_file + ' ready' )
rescue Exception => e
	UI.messagebox( 'Could not write to ' + html_file + "\n\n" +
		'Reason: ' + e.to_s )
end
=begin
=end

end # module Keycuts

# end of /t/keycuts_writer.rb